; GIMP - The GNU Image Manipulation Program
; Copyright (C) 1995 Spencer Kimball and Peter Mattis
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;
;
; 
; Creates a "melting" animation of the active layer in an image as described in my tutorial here:
;
; http://www.gimpdome.com/index.php?topic=3154.0
; The animation may be saved with the gif-plug-in.
;
; The script can be found under Filters > Animation > Melt Me...
;
; Script updated on October 3, 2008 to work with GIMP 2.6

; Define the Function

(define (script-fu-melt-me-anim 	img		 
				drawable
				counter
				inSpeed
				pause1
				pause2
				addBorder
				borderWidth
				borderColor				
				blendOpt
				)

; Declare the Variables

	(let* (
		    	(speed 0)
			(width 0)
			(height 0)	
			(theSelection 0)
			(newImage 0)		    
			(activeLayer 0)
			(activeLayerCopy 0)
			(activeLayerHolder 0)
			(floatingSelection 0)
			(borderBlendMode
				(cond 
				(( equal? blendOpt 0 ) LAYER-MODE-NORMAL-LEGACY)
				(( equal? blendOpt 1 ) LAYER-MODE-DISSOLVE)
				(( equal? blendOpt 2 ) LAYER-MODE-MULTIPLY-LEGACY)
				(( equal? blendOpt 3 ) LAYER-MODE-DIVIDE-LEGACY)
				(( equal? blendOpt 4 ) LAYER-MODE-SCREEN-LEGACY)
				(( equal? blendOpt 5 ) LAYER-MODE-OVERLAY-LEGACY)
				(( equal? blendOpt 6 ) LAYER-MODE-DODGE-LEGACY)
				(( equal? blendOpt 7 ) LAYER-MODE-BURN-LEGACY)
				(( equal? blendOpt 8 ) LAYER-MODE-HARDLIGHT-LEGACY)
				(( equal? blendOpt 9 ) LAYER-MODE-SOFTLIGHT-LEGACY)
				(( equal? blendOpt 10 ) LAYER-MODE-GRAIN-EXTRACT-LEGACY)
				(( equal? blendOpt 11 ) LAYER-MODE-GRAIN-MERGE-LEGACY)
				(( equal? blendOpt 12 ) LAYER-MODE-DIFFERENCE-LEGACY)
				(( equal? blendOpt 13 ) LAYER-MODE-ADDITION-LEGACY)
				(( equal? blendOpt 14 ) LAYER-MODE-SUBTRACT-LEGACY)
				(( equal? blendOpt 15 ) LAYER-MODE-DARKEN-ONLY-LEGACY)
				(( equal? blendOpt 16 ) LAYER-MODE-LIGHTEN-ONLY-LEGACY)
				(( equal? blendOpt 17 ) LAYER-MODE-HUE)
				(( equal? blendOpt 18 ) LAYER-MODE-SATURATION)
				(( equal? blendOpt 19 ) LAYER-MODE-COLOR)
				(( equal? blendOpt 20 ) LAYER-MODE-VALUE)
				)	
			)
			(displaceLayer 0)
		    	(displaceLayerCopy 0)
			(layerName 0)
		    	(remainingFrames counter 0)
			(transparentLayer 0)
			(offset 0)
			(step 0)
			(frameNum 1)
	            	(borderLayer 0)
			(totalFrames counter)
		)

(gimp-context-push)


(set! activeLayer (car (gimp-image-get-active-layer img)))
(set! theSelection (car (gimp-selection-save img)))
(gimp-selection-none img)
(set! width (car (gimp-drawable-width activeLayer)))
(set! height (car (gimp-drawable-height activeLayer)))
(set! activeLayer (car (gimp-edit-copy activeLayer)))
(set! newImage (car (gimp-image-new width height RGB)))
(gimp-image-undo-disable newImage)
(set! activeLayerHolder (car (gimp-layer-new newImage width height RGBA-IMAGE 	"Active Layer" 100 LAYER-MODE-NORMAL-LEGACY)))
(gimp-drawable-fill activeLayerHolder FILL-TRANSPARENT)	
(gimp-image-add-layer newImage activeLayerHolder -1)
(set! floatingSelection (car (gimp-edit-paste activeLayerHolder TRUE))) 	
(gimp-floating-sel-anchor floatingSelection)	
(set! activeLayer (car (gimp-image-get-active-layer newImage)))
(set! displaceLayer (car (gimp-layer-new newImage width height RGBA-IMAGE "Displace Layer" 100 LAYER-MODE-NORMAL-LEGACY)))
(gimp-image-add-layer newImage displaceLayer -1) 
(plug-in-solid-noise RUN-NONINTERACTIVE newImage displaceLayer 0 0 (rand) (rand) 4.0 4.0)
(set! displaceLayerCopy (car (gimp-layer-copy displaceLayer TRUE)))
(gimp-image-add-layer newImage displaceLayerCopy -1)
(plug-in-solid-noise RUN-NONINTERACTIVE newImage displaceLayerCopy 0 0 (rand) (rand) 4.0 4.0)
(gimp-layer-set-mode displaceLayerCopy LAYER-MODE-SCREEN-LEGACY)
(gimp-image-merge-down newImage displaceLayerCopy CLIP-TO-IMAGE)
(set! displaceLayer (car (gimp-image-get-active-layer newImage)))
(gimp-brightness-contrast displaceLayer 0 10)


(set! offset (* 2 height))
(set! step (/ (* 4 height) counter))

(while (> counter 0)

(set! speed inSpeed)	

(if (= frameNum 1)
    (begin
    (set! speed pause1)
    (set! transparentLayer (car (gimp-layer-copy activeLayer TRUE)))
    (gimp-image-add-layer newImage transparentLayer -1)
    (gimp-drawable-fill transparentLayer FILL-TRANSPARENT)	
    (set! layerName (string-append "Frame " (number->string frameNum) " (" (number->string speed) "ms)" " (replace)"))
    (gimp-drawable-set-name transparentLayer layerName)
    (set! frameNum (+ frameNum 1))
    (set! speed inSpeed)
	
	(if (= addBorder TRUE)
    	    (begin
    	    (set! borderLayer (car (gimp-layer-copy activeLayer TRUE)))
	    (gimp-image-add-layer newImage borderLayer -1)
	    (gimp-selection-all newImage)
	    (gimp-context-set-foreground borderColor)
	    (gimp-drawable-fill borderLayer FILL-FOREGROUND)
	    (gimp-selection-shrink newImage borderWidth)
	    (gimp-edit-clear borderLayer)
	    (gimp-layer-set-mode borderLayer borderBlendMode)
	    (gimp-selection-none newImage)
	    (gimp-image-merge-down newImage borderLayer CLIP-TO-IMAGE) 
	    )
	)
    )
)

(if (= frameNum (trunc (/ totalFrames 2)))
    (begin
    (set! speed pause2)
    (set! offset 0)
    (set! counter (- counter 1))
    )
)


(set! activeLayerCopy (car (gimp-layer-copy activeLayer TRUE)))
(gimp-image-add-layer newImage activeLayerCopy -1)

(if (= frameNum 1)
(plug-in-displace RUN-NONINTERACTIVE newImage activeLayerCopy 0 0 0 0  displaceLayer displaceLayer 3)
)

(plug-in-displace RUN-NONINTERACTIVE newImage activeLayerCopy 0 offset 0 1  displaceLayer displaceLayer 3)


(set! layerName (string-append "Frame " (number->string frameNum) " (" (number->string speed) "ms)" " (replace)"))
(gimp-drawable-set-name activeLayerCopy layerName)

(set! speed inSpeed)
(set! offset (- offset step))
(set! frameNum (+ frameNum 1))
(set! counter (- counter 1))



; Adds a border layer if chosen by the user with the values set

(if (= addBorder TRUE)
    (begin
    (set! borderLayer (car (gimp-layer-copy activeLayer TRUE)))
    (gimp-image-add-layer newImage borderLayer -1)
    (gimp-selection-all newImage)
    (gimp-context-set-foreground borderColor)
    (gimp-drawable-fill borderLayer FILL-FOREGROUND)
    (gimp-selection-shrink newImage borderWidth)
    (gimp-edit-clear borderLayer)
    (gimp-layer-set-mode borderLayer borderBlendMode)
    (gimp-selection-none newImage)
    (gimp-image-merge-down newImage borderLayer CLIP-TO-IMAGE) 
    )
)

) ; goes with while

; Removes the original layer from the stack because it's no longer needed  
  
(gimp-image-remove-layer newImage activeLayer) 
(gimp-image-remove-layer newImage displaceLayer) 
(gimp-image-undo-enable newImage)

(gimp-selection-load theSelection)
(gimp-image-remove-channel img theSelection)

(gimp-context-pop)

(gimp-displays-flush)

; Displays the final animation

(gimp-display-new newImage)
(gimp-image-set-active-layer img drawable)

  )
)

(script-fu-register "script-fu-melt-me-anim"
  "<Image>/Filters/Animation/Animators/Melt Me..."
  "Create an animated melt effect"
  "Art Wade"
  "Art Wade"
  "October 3, 20082008"
  "RGB*"
  SF-IMAGE       	"Image" 0
  SF-DRAWABLE    	"Drawable" 0
  SF-ADJUSTMENT 	"Number of frames" '(50 15 50 1 10 0 1)
  SF-ADJUSTMENT 	"Animation Speed (in ms)" '(100 10 500 1 1 0 1)
  SF-ADJUSTMENT	  	"First Frame Delay Time (in ms)" '(2000 10 20000 1 1 0 1)
  SF-ADJUSTMENT	  	"Middle Frame Delay Time (in ms)" '(2000 10 20000 1 1 0 1)
  SF-TOGGLE       	"Add Border?" FALSE
  SF-ADJUSTMENT   	"Border Width" '(1 1 3 1 1 0 1)
  SF-COLOR        	"Border Color" '(255 255 255)  
  SF-OPTION		"Border Blend Mode" 		'("LAYER-MODE-NORMAL-LEGACY" 
							"LAYER-MODE-DISSOLVE"
							"LAYER-MODE-MULTIPLY-LEGACY"
							"LAYER-MODE-DIVIDE-LEGACY"
							"LAYER-MODE-SCREEN-LEGACY"
							"LAYER-MODE-OVERLAY-LEGACY"
							"LAYER-MODE-DODGE-LEGACY"
							"LAYER-MODE-BURN-LEGACY"
							"LAYER-MODE-HARDLIGHT-LEGACY"
							"LAYER-MODE-SOFTLIGHT-LEGACY"
							"LAYER-MODE-GRAIN-EXTRACT-LEGACY"
							"LAYER-MODE-GRAIN-MERGE-LEGACY"														"LAYER-MODE-DIFFERENCE-LEGACY"
							"LAYER-MODE-ADDITION-LEGACY"
							"LAYER-MODE-SUBTRACT-LEGACY"
							"LAYER-MODE-DARKEN-ONLY-LEGACY"
							"LAYER-MODE-LIGHTEN-ONLY-LEGACY"
							"LAYER-MODE-HUE"
							"LAYER-MODE-SATURATION"
							"LAYER-MODE-COLOR"
							"LAYER-MODE-VALUE")
)


                         
